extern const char *cmd2lex(const char *);
extern int dequote_txt(USTRING *, const char *,  size_t);
extern void lex_error(const char *, int);

enum LEX_TYPE {
	LEX_SPACE = 10,			/* white space */

	LEX_TEXT = 20, LEX_TEXT2, 	/* regular text */
		LEX_QMARK,				/* quotation marks */
		LEX_MARK,				/* other marks */
		LEX_VAR,				/* $ symbol (variable substitution) */

	LEX_MATCH_ONE = 30, 	/* * */
		LEX_MATCH_ANY,		/* ? */
								/* list [ .... ] */
		LEX_MATCH_BEGIN, 		/* [ */
		LEX_MATCH_INVERSE,		/* ^ or ! immediately after [ */
		LEX_MATCH_END,			/* ] */
		LEX_MATCH_LIST,			/* character from the list */
		LEX_MATCH_RANGE_FROM,	/* first char of a range a-z */
		LEX_MATCH_RANGE_TO,		/* last char of a range a-z */

	LEX_IO = 40,	/* > or < */
	LEX_CMDSEP,		/* ; & | ( or opening ` */
	LEX_OTHER,		/* } closing ` etc. */

	LEX_BEGIN = 50,	 /* begin of text */
	/* end of text + exit code */
	LEX_END_OK,		/* ok */
	LEX_END_ERR_BQ,	/* trailing backslash */
	LEX_END_ERR_SQ,	/* open single quote */
	LEX_END_ERR_DQ	/* open double quote */
};

#define LEX_TYPE(X)		((X) / 10)

#define LEX_TYPE_SPACE		1
#define LEX_TYPE_WORD		2
#define LEX_TYPE_MATCH		3
#define LEX_TYPE_SPECIAL	4
#define LEX_TYPE_END		5

#define IS_LEX_SPACE(X)		(LEX_TYPE(X) == LEX_TYPE_SPACE)
#define IS_LEX_WORD(X)		(LEX_TYPE(X) == LEX_TYPE_WORD || LEX_TYPE(X) == LEX_TYPE_MATCH)
#define IS_LEX_MATCH(X)		(LEX_TYPE(X) == LEX_TYPE_MATCH)
#define IS_LEX_SPECIAL(X)	(LEX_TYPE(X) == LEX_TYPE_SPECIAL)
#define IS_LEX_END(X)		(LEX_TYPE(X) == LEX_TYPE_END)

#define IS_LEX_EMPTY(X)		(LEX_TYPE(X) == LEX_TYPE_END || LEX_TYPE(X) == LEX_TYPE_SPACE)
#define IS_LEX_CMDSEP(X)	((X) == LEX_CMDSEP || (X) == LEX_BEGIN)
#define IS_LEX_TEXT(X)		((X) == LEX_TEXT || (X) == LEX_TEXT2)
